@@ -48,6 +48,9 @@ from crate.client.compat import raw_input
 from .tabulate import TableFormat, Line, DataRow, tabulate
 
 
+MAX_HISTORY_LENGTH = 10000
+
+
 crate_fmt = TableFormat(lineabove=Line("+", "-", "+", "+"),
                         linebelowheader=Line("+", "-", "+", "+"),
                         linebetweenrows=None,
@@ -395,27 +398,27 @@ def get_stdin():
     # use select.select to check if input is available
     # otherwise sys.stdin would block
     partial_lines = []
-    stdin_data = []
     delim = CrateCmd.line_delimiter
     while sys.stdin in select.select([sys.stdin], [], [], 0)[0]:
         line = sys.stdin.readline()
         if line:
-            if line.lstrip().startswith('--'):
+            line = line.strip()
+            if line.startswith('--'):
                 continue
-            if not line.endswith(delim):
-                partial_lines.append(line)
-            elif not partial_lines:
-                for single_cmd in line.rstrip(delim).split(delim):
-                    stdin_data.append(single_cmd)
+            if line.endswith(delim):
+                line = line.rstrip(delim)
+                if partial_lines:
+                    yield ''.join(partial_lines) + line
+                    partial_lines = []
+                else:
+                    yield line
             else:
                 partial_lines.append(line)
         else:
             if partial_lines:
-                lines = "".join(partial_lines)
-                for single_cmd in lines.rstrip(delim).split(delim):
-                    stdin_data.append(single_cmd)
+                yield ''.join(partial_lines) + line
             break
-    return stdin_data
+    return
 
 
 def main():
@@ -424,6 +427,7 @@ def main():
     # optionally read and write history file
     if _has_readline:
         history_file_path = args.history
+        readline.set_history_length(MAX_HISTORY_LENGTH)
         if not os.path.exists(USER_DATA_DIR):
             os.makedirs(USER_DATA_DIR)
         try:
@@ -435,15 +439,18 @@ def main():
     cmd.do_connect(args.hosts)
     # select.select on sys.stdin doesn't work on windows
     # so currently there is no pipe support
+    done = False
     if os.name == 'posix':
         stdin_data = get_stdin()
     if args.command:
         for single_cmd in args.command.split(CrateCmd.line_delimiter):
             cmd.onecmd(single_cmd)
+        done = True
     elif stdin_data:
         for single_cmd in stdin_data:
             cmd.onecmd(single_cmd)
-    else:
+            done = True
+    if not done:
         try:
             cmd.cmdloop()
         except KeyboardInterrupt:
